#!/usr/bin/env python
""" MultiThreaded scraping of ENRs and conversion to output formats

To run: 

python -m nrc_scrape.bulk_downloader --start_year YYYY --end_year YYYY --threads 16

"""
import os
import argparse
from nrc_scrape.definitions.sessions import session_factory
import requests
import datetime
from queue import Queue
from threading import Thread, get_ident
from typing import List

import pandas as pd

from nrc_scrape.definitions.classes import EventNotificationReport
from nrc_scrape.main import generate_nrc_event_report_urls
from nrc_scrape import fl, sl, el, ol, session


def urls2list(urls: dict) -> List[str]:
    """generates a list of EventNotificationReport objects from a list of urls"""
    urls_l = []
    for _, months in urls.items():
        for _, murls in months.items():
            for url in murls:
                urls_l.append(url)
    return urls_l


class EnrDownloadWorker(Thread):
    """ worker thread to get event report url"""

    def __init__(self, url_queue, enr_queue, session: requests.Session):
        Thread.__init__(self)
        self.url_queue = url_queue
        self.enr_queue = enr_queue
        self.s = session

    def run(self):
        while True:
            url = self.url_queue.get()
            ol.debug(f"{self.ident} downloading {url}.")
            try:
                enr = EventNotificationReport.from_url(url, self.s)
                self.enr_queue.put(enr)
                sl.info(enr)
            except:
                el.info(url)
            finally:
                self.url_queue.task_done()


def download(year):
    """ spins up worker threads, downloads and parses a year of ENRs into queue, returns queue"""

    ol.info(f"Preparing download urls for {year}")

    urls: dict = generate_nrc_event_report_urls(year, year)

    ol.info(f"Finished getting urls.")

    urls = urls2list(urls)  # type: ignore
    ol.info(f"{len(urls)} urls to download")

    enr_urls_queue = Queue()
    enr_queue = Queue()

    for i in range(16):
        ol.info(f"Starting download worker {i}")
        worker = EnrDownloadWorker(enr_urls_queue, enr_queue, session_factory())
        worker.daemon = True
        worker.start()

    for url in urls:
        enr_urls_queue.put(url)

    ol.info(f"~{enr_urls_queue.qsize()} enr urls to download.")

    enr_urls_queue.join()

    return enr_queue


def main():
    """Main function for bulk downloads of ENRs and converts to csv"""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--start_year",
        type=int,
        help="First year to download event reports.",
        default=2004,
    )
    parser.add_argument(
        "--threads", type=int, help="Number of concurrent download threads.", default=16
    )
    parser.add_argument(
        "--end_year",
        type=int,
        help="Last year to download event reports.",
        default=datetime.datetime.now().year,
    )

    parser.add_argument(
        "--data_directory",
        type=str,
        help="directory to place data files",
        default="./data",
    )
    args = parser.parse_args()
    start_year: int = args.start_year
    end_year: int = args.end_year
    data_directory: str = args.data_directory

    ol.info(f"Starting download for {start_year} to {end_year}.")

    for year in range(start_year, end_year + 1):
        q = download(year)
        enrs = []
        while not q.empty():
            enrs.append(q.get())

        enr_dfs = []
        for enr in enrs:
            try:
                enr_df = enr.to_dataframe()
                enr_dfs.append(enr_df)
            except:
                pass

        df = pd.concat(enr_dfs)
        df.to_csv(os.path.join(data_directory, str(year) + ".csv"))


if __name__ == "__main__":
    main()


import logging
import os

logs = ["success_log", "error_log", "fof_log", "op_log"]

if not os.path.exists("logs"):
    os.mkdir("logs")

for log in logs:

    logger = logging.getLogger(log)
    logger.setLevel(logging.DEBUG)

    fh = logging.FileHandler(f"./logs/{log}.log")
    fh.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    formatter = logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )

    fh.setFormatter(formatter)
    ch.setFormatter(formatter)

    logger.addHandler(fh)
    logger.addHandler(ch)

sl = logging.getLogger("success_log")
el = logging.getLogger("error_log")
fl = logging.getLogger("fof_log")
ol = logging.getLogger("op_log")



# make logs accessible from top level module name
from .logs_setup import el, fl, ol, sl

from .definitions.sessions import session_factory

# global session for connecting for urls, multithreaded scraper should use the factory
session = session_factory()


import argparse
import datetime
import json
from typing import Dict, List

import requests
from bs4 import BeautifulSoup

from nrc_scrape import ol

nrc_events_page_url = (
    "https://www.nrc.gov/reading-rm/doc-collections/event-status/event/"
)

from nrc_scrape import session


def get_events_toc_html() -> bytes:
    """Gets the main events table of content page and returns html

    Returns:
        str: html of the events toc page
    """
    req = session.get(nrc_events_page_url)
    ol.debug(f"Got events page from {nrc_events_page_url}.")
    return req.content


def get_main_sub(page_html):
    """Filters to main sub div from events page

    Args:
        page_html ([type]): [description]

    Returns:
        [type]: [description]
    """
    return page_html.find(id="mainSub")


def get_links_from_year_para(year_para):
    """Get all the links associatged with each year from the TOC events page.

    Args:
        year_para ([type]): [description]

    Returns:
        [type]: [description]
    """
    return year_para.find_all("a")


def get_nrc_url(url, session):
    req = None
    attempts = 0
    while req is None and attempts < 5:
        req = session.get(url, timeout=3)
    req.raise_for_status()
    if not req:
        raise ValueError("Unable to fetch url")
    return req


def get_year_urls(year_paras, start_year, end_year):
    links = []
    for year_para in year_paras:
        year_links = get_links_from_year_para(year_para)
        for year_link in year_links:
            year = int(year_link.text)
            if year >= start_year and year <= end_year:
                url = f'http://www.nrc.gov{year_link.get("href")}'
                links.append(url)
    return links


def get_tag_name_property(tag, name):
    return tag.attrs(name)


def make_urls(
    start_year: int = 2003, end_year: int = datetime.datetime.now().year
) -> Dict[str, Dict[str, List[str]]]:
    """Generates urls needed to access NRC event reports.

    Args:
        start_year (int, optional): Start year of Event Report, needs to be > 2003 for formatting change. Defaults to 2003.
        end_year (int, optional): End year of Event Reports. Defaults to datetime.datetime.now().year.

    Returns:
        Dict[year, List[Urls]] : Dict of urls by year.
    """
    events_html = get_events_toc_html()
    page_html = BeautifulSoup(events_html, "html.parser")
    year_paras = get_main_sub(page_html).find_all_next("p")[1:4]
    yurls = get_year_urls(year_paras, start_year, end_year)

    all_links = {}

    for yurl in yurls:
        ol.debug(f"Getting {yurl}.")
        year_links = {}

        page_html = get_nrc_url(yurl, session).content
        parsed = BeautifulSoup(page_html, "html.parser")
        main_sub = get_main_sub(parsed)

        # month listing is first paragraph in main sub
        months = main_sub.find_next("p")

        # get month anchor links
        months = months.find_all("a")

        # get the month name from anchor link definition
        months = [month.get("href")[1:] for month in months]

        # get month name that have a url for this year
        months_with_url = [
            x.find("a")
            for x in main_sub.find_all("h3")
            if x.find("a").attrs.get("name") in months
        ]

        # months broken up by h3 tags
        month_h3s = main_sub.find_all("h3")

        # filter h3s to just our months listed
        for month in month_h3s:
            if month.find("a") in months_with_url:
                # next p tag holds the links
                try:
                    year_links[month.text] = [
                        yurl + enr_url.get("href")
                        for enr_url in month.find_next("p").find_all(
                            "a", recursive=False
                        )
                    ]
                except:
                    Warning(f"No {yurl}")
        # year is end of link
        year_num = yurl[-5:-1]
        all_links[year_num] = year_links

    return all_links


def main():
    """Write urls into --url_json_file."""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--url_json_file", type=str, help="Previously created JSON file of URLs "
    )
    url_json_file = parser.parse_args().url_json_file
    urls = make_urls()
    with open(url_json_file, "w") as outfile:
        json.dump(urls, outfile)


if __name__ == "__main__":
    main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="nrc-scrape", # Replace with your own username
    version="0.0.5",
    author="Bradley Fox",
    author_email="bradfox2@gmail.com",
    description="Python package to scrape NRC Event Reports.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/bradfox2/nrc-scrape",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=['beautifulsoup4==4.8.2',
                        'numpy==1.18.1',
                        'pandas==1.0.1',
                        'requests==2.23.0',
                        'python-dateutil==2.8.1'],
    python_requires='>=3.7',
)


from __future__ import annotations

import datetime
import json
import sys

import requests
from requests.exceptions import HTTPError

from nrc_scrape.definitions.classes import EventNotificationReport

from nrc_scrape import url_maker, fl, sl, el, ol


def generate_nrc_event_report_urls(
    start_year=2003, end_year: int = datetime.date.today().year, url_json_file=None
):
    """Constructs a list of nrc event report page urls from year start to years end.  Format changes < 2003, not able to parse old style.

    Args:
        start_year (int, optional):  Defaults to 2003.
        end_year (int, optional): Defaults to datetime.date.today().year.
        url_json_file (str, optional): Load URLs from this file. Defaults to None.

    Returns:
        [type]: [description]
    """
    if url_json_file:
        with open(url_json_file, "r") as file:
            urls = json.load(file)
        return urls
    else:
        return url_maker.make_urls(start_year=start_year, end_year=end_year)


def fetch_enrs(urls, session: requests.Session):
    """generates a list of EventNotificationReport objects from a list of urls"""

    error_list = []
    enrs = []
    four_oh_fours = []
    nurls = len(urls)
    for idx, url in enumerate(urls):
        ol.info(f"{idx}/{nurls}, {url}")
        try:
            en = EventNotificationReport.from_url(url, session)
            enrs.append(en)
            sl.info(en)
        except HTTPError:
            four_oh_fours.append(url)
            fl.info(url)
            next
        except:
            error_list.append((url, sys.exc_info()[0]))
            el.info((url, sys.exc_info()[0]))
            print("ERROR!")
            next

    print(f"{len(enrs)}:OK, {len(error_list)}:Failed, {len(four_oh_fours)}:404s")
    return enrs, error_list, four_oh_fours
